# coding: utf-8

"""
    Assetic Integration SDK
    Initiates API wrappers and tools to manage integration workflows
"""

from __future__ import absolute_import

import os
import logging
import six
try:
    import configparser as configparser
except:
    import ConfigParser as configparser
from .api_client import ApiClient
from .configuration import Configuration
class AsseticSDK(object):
    """
    class to initialise sdk and api methods
    """
    def __init__(self,inifile=None,logfile=None,loglevelname=None):
        """
        Constructor of the class.
        Param inifile: the file name (including path) of the ini file
        (host, username, api_key).
        If none then will look in local folder for assetic.ini
        ,else look in environment variables for asseticSDKhost, asseticSDKusername
        ,asseticSDKapi_key
        Param logfile: the file name (including path) of the log file.
        If None then no logfile will be created
        Param loglevelname: set as a valid logging level description e.g. INFO
        """
        host = None
        if inifile == None:
            ##look in current folder for 'assetic.ini'
            inifile = os.path.join(os.getcwd(), 'assetic.ini')
        if os.path.isfile(inifile) == True:
            host, username, api_key = self.get_credentials_via_file(inifile)
        else:
            host, username, api_key = self.get_credentials_via_env()

        if host == None:
            message = """Configuration file not found.
                Expecting assetic.ini in either the current folder,
                or set via module init as the first parameter,
                or set in the following environment variables:
                ASSETICHOST: the base URL
                ASSETICUSER: the Assetic username
                ASSETICKEY for the token"""
            raise ValueError(message)
            
        config = Configuration()
        config.host = host
        config.username = username
        config.password = api_key
        auth = config.get_basic_auth_token()

        if logfile != None:
            config.logger_file = logfile
        loglevel = self.verify_log_level(loglevelname)
        config.packagelogger.setLevel(loglevel)
        if config.packagelogger.level == 10:
            config.debug = True

        ##Assetic SDK client instance      
        self.client = ApiClient(config.host,"Authorization",auth)
        self.client.user_agent="Assetic_Python_SDK"
        ##Asset SDK client instance for docs
        self.client_for_docs = ApiClient(config.host,"Authorization", auth ,
                                None,'Content-Disposition')
        self.client_for_docs.user_agent="Assetic_Python_SDK"
        ##Set the defualt client against config
        config.api_client = self.client

        ##reference config in self so we can use it
        self.conf = config
        
    @property
    def logger(self):
        """
        Gets the logger.  Allows referencing of logger and further customisation
        """
        return self.conf.packagelogger
    
        
    def get_credentials_via_file(self,inifile):
        """
        get credentials from ini file
        param inifile: ini file with host, username, api_key 
        Returns: tuple containing:
        host: assetic host
        username: username
        api_key: authentication token
        """
        settings = configparser.ConfigParser()
        settings.read(inifile)

        host = settings.get('environment', 'url')
        username = settings.get('auth', 'username')
        password = settings.get('auth', 'api_key')        
        return host,username,password

    def get_credentials_via_env(self):
        """
        get credentials from environment variables. Looks for ASSETICHOST for
        the base URL, ASSETICUSER for the username, ASSETICKEY for the token
        Returns: tuple containing:
        host: assetic host
        username: username
        api_key: authentication token
        """
        host = None
        username = None
        password = None
        if 'ASSETICHOST' in os.environ:
            host = os.environ['ASSETICHOST']
        if 'ASSETICUSER' in os.environ:
            username = os.environ['ASSETICUSER']
        if 'ASSETICKEY' in os.environ:
            password = os.environ['ASSETICKEY']            
        return host,username,password
    
    def verify_log_level(self,loglevelname):
        """
        verify loglevel is valid for given text description
        needs to be a valid logging name, else set to zero (no logging)
        param loglevelname: the name of the logging level to test & set
        Returns: loglevel (int)
        """
        ##make sure a string is being tested.
        ##If an int assume user is setting the level directly
        if isinstance(loglevelname, six.integer_types):
            return loglevelname
        elif isinstance(loglevelname, six.string_types): 
            try:
                loglevel = getattr(logging,loglevelname.upper())
            except AttributeError as ex:
                raise ValueError(str(ex))
                return 0
            return loglevelname.upper()
        else:
            return 0
        
